var htmlConfig = {
  autoSelfClosers: {
    "area": true,
    "base": true,
    "br": true,
    "col": true,
    "command": true,
    "embed": true,
    "frame": true,
    "hr": true,
    "img": true,
    "input": true,
    "keygen": true,
    "link": true,
    "meta": true,
    "param": true,
    "source": true,
    "track": true,
    "wbr": true,
    "menuitem": true
  },
  implicitlyClosed: {
    "dd": true,
    "li": true,
    "optgroup": true,
    "option": true,
    "p": true,
    "rp": true,
    "rt": true,
    "tbody": true,
    "td": true,
    "tfoot": true,
    "th": true,
    "tr": true
  },
  contextGrabbers: {
    "dd": { "dd": true, "dt": true },
    "dt": { "dd": true, "dt": true },
    "li": { "li": true },
    "option": { "option": true, "optgroup": true },
    "optgroup": { "optgroup": true },
    "p": {
      "address": true,
      "article": true,
      "aside": true,
      "blockquote": true,
      "dir": true,
      "div": true,
      "dl": true,
      "fieldset": true,
      "footer": true,
      "form": true,
      "h1": true,
      "h2": true,
      "h3": true,
      "h4": true,
      "h5": true,
      "h6": true,
      "header": true,
      "hgroup": true,
      "hr": true,
      "menu": true,
      "nav": true,
      "ol": true,
      "p": true,
      "pre": true,
      "section": true,
      "table": true,
      "ul": true
    },
    "rp": { "rp": true, "rt": true },
    "rt": { "rp": true, "rt": true },
    "tbody": { "tbody": true, "tfoot": true },
    "td": { "td": true, "th": true },
    "tfoot": { "tbody": true },
    "th": { "td": true, "th": true },
    "thead": { "tbody": true, "tfoot": true },
    "tr": { "tr": true }
  },
  doNotIndent: { "pre": true },
  allowUnquoted: true,
  allowMissing: true,
  caseFold: true
};
var xmlConfig = {
  autoSelfClosers: {},
  implicitlyClosed: {},
  contextGrabbers: {},
  doNotIndent: {},
  allowUnquoted: false,
  allowMissing: false,
  allowMissingTagName: false,
  caseFold: false
};
function mkXML(parserConfig) {
  var config2 = {};
  var defaults = parserConfig.htmlMode ? htmlConfig : xmlConfig;
  for (var prop in defaults)
    config2[prop] = defaults[prop];
  for (var prop in parserConfig)
    config2[prop] = parserConfig[prop];
  var type, setStyle;
  function inText(stream, state) {
    function chain(parser) {
      state.tokenize = parser;
      return parser(stream, state);
    }
    var ch = stream.next();
    if (ch == "<") {
      if (stream.eat("!")) {
        if (stream.eat("[")) {
          if (stream.match("CDATA["))
            return chain(inBlock("atom", "]]>"));
          else
            return null;
        } else if (stream.match("--")) {
          return chain(inBlock("comment", "-->"));
        } else if (stream.match("DOCTYPE", true, true)) {
          stream.eatWhile(/[\w\._\-]/);
          return chain(doctype(1));
        } else {
          return null;
        }
      } else if (stream.eat("?")) {
        stream.eatWhile(/[\w\._\-]/);
        state.tokenize = inBlock("meta", "?>");
        return "meta";
      } else {
        type = stream.eat("/") ? "closeTag" : "openTag";
        state.tokenize = inTag;
        return "angleBracket";
      }
    } else if (ch == "&") {
      var ok;
      if (stream.eat("#")) {
        if (stream.eat("x")) {
          ok = stream.eatWhile(/[a-fA-F\d]/) && stream.eat(";");
        } else {
          ok = stream.eatWhile(/[\d]/) && stream.eat(";");
        }
      } else {
        ok = stream.eatWhile(/[\w\.\-:]/) && stream.eat(";");
      }
      return ok ? "atom" : "error";
    } else {
      stream.eatWhile(/[^&<]/);
      return null;
    }
  }
  inText.isInText = true;
  function inTag(stream, state) {
    var ch = stream.next();
    if (ch == ">" || ch == "/" && stream.eat(">")) {
      state.tokenize = inText;
      type = ch == ">" ? "endTag" : "selfcloseTag";
      return "angleBracket";
    } else if (ch == "=") {
      type = "equals";
      return null;
    } else if (ch == "<") {
      state.tokenize = inText;
      state.state = baseState;
      state.tagName = state.tagStart = null;
      state.tokenize(stream, state);
      return "invalid";
    } else if (/[\'\"]/.test(ch)) {
      state.tokenize = inAttribute(ch);
      state.stringStartCol = stream.column();
      return state.tokenize(stream, state);
    } else {
      stream.match(/^[^\s\u00a0=<>\"\']*[^\s\u00a0=<>\"\'\/]/);
      return "word";
    }
  }
  function inAttribute(quote) {
    var closure = function(stream, state) {
      while (!stream.eol()) {
        if (stream.next() == quote) {
          state.tokenize = inTag;
          break;
        }
      }
      return "string";
    };
    closure.isInAttribute = true;
    return closure;
  }
  function inBlock(style, terminator) {
    return function(stream, state) {
      while (!stream.eol()) {
        if (stream.match(terminator)) {
          state.tokenize = inText;
          break;
        }
        stream.next();
      }
      return style;
    };
  }
  function doctype(depth) {
    return function(stream, state) {
      var ch;
      while ((ch = stream.next()) != null) {
        if (ch == "<") {
          state.tokenize = doctype(depth + 1);
          return state.tokenize(stream, state);
        } else if (ch == ">") {
          if (depth == 1) {
            state.tokenize = inText;
            break;
          } else {
            state.tokenize = doctype(depth - 1);
            return state.tokenize(stream, state);
          }
        }
      }
      return "meta";
    };
  }
  function lower(tagName) {
    return tagName && tagName.toLowerCase();
  }
  function Context(state, tagName, startOfLine) {
    this.prev = state.context;
    this.tagName = tagName || "";
    this.indent = state.indented;
    this.startOfLine = startOfLine;
    if (config2.doNotIndent.hasOwnProperty(tagName) || state.context && state.context.noIndent)
      this.noIndent = true;
  }
  function popContext(state) {
    if (state.context)
      state.context = state.context.prev;
  }
  function maybePopContext(state, nextTagName) {
    var parentTagName;
    while (true) {
      if (!state.context) {
        return;
      }
      parentTagName = state.context.tagName;
      if (!config2.contextGrabbers.hasOwnProperty(lower(parentTagName)) || !config2.contextGrabbers[lower(parentTagName)].hasOwnProperty(lower(nextTagName))) {
        return;
      }
      popContext(state);
    }
  }
  function baseState(type2, stream, state) {
    if (type2 == "openTag") {
      state.tagStart = stream.column();
      return tagNameState;
    } else if (type2 == "closeTag") {
      return closeTagNameState;
    } else {
      return baseState;
    }
  }
  function tagNameState(type2, stream, state) {
    if (type2 == "word") {
      state.tagName = stream.current();
      setStyle = "tag";
      return attrState;
    } else if (config2.allowMissingTagName && type2 == "endTag") {
      setStyle = "angleBracket";
      return attrState(type2, stream, state);
    } else {
      setStyle = "error";
      return tagNameState;
    }
  }
  function closeTagNameState(type2, stream, state) {
    if (type2 == "word") {
      var tagName = stream.current();
      if (state.context && state.context.tagName != tagName && config2.implicitlyClosed.hasOwnProperty(lower(state.context.tagName)))
        popContext(state);
      if (state.context && state.context.tagName == tagName || config2.matchClosing === false) {
        setStyle = "tag";
        return closeState;
      } else {
        setStyle = "error";
        return closeStateErr;
      }
    } else if (config2.allowMissingTagName && type2 == "endTag") {
      setStyle = "angleBracket";
      return closeState(type2, stream, state);
    } else {
      setStyle = "error";
      return closeStateErr;
    }
  }
  function closeState(type2, _stream, state) {
    if (type2 != "endTag") {
      setStyle = "error";
      return closeState;
    }
    popContext(state);
    return baseState;
  }
  function closeStateErr(type2, stream, state) {
    setStyle = "error";
    return closeState(type2, stream, state);
  }
  function attrState(type2, _stream, state) {
    if (type2 == "word") {
      setStyle = "attribute";
      return attrEqState;
    } else if (type2 == "endTag" || type2 == "selfcloseTag") {
      var tagName = state.tagName, tagStart = state.tagStart;
      state.tagName = state.tagStart = null;
      if (type2 == "selfcloseTag" || config2.autoSelfClosers.hasOwnProperty(lower(tagName))) {
        maybePopContext(state, tagName);
      } else {
        maybePopContext(state, tagName);
        state.context = new Context(state, tagName, tagStart == state.indented);
      }
      return baseState;
    }
    setStyle = "error";
    return attrState;
  }
  function attrEqState(type2, stream, state) {
    if (type2 == "equals")
      return attrValueState;
    if (!config2.allowMissing)
      setStyle = "error";
    return attrState(type2, stream, state);
  }
  function attrValueState(type2, stream, state) {
    if (type2 == "string")
      return attrContinuedState;
    if (type2 == "word" && config2.allowUnquoted) {
      setStyle = "string";
      return attrState;
    }
    setStyle = "error";
    return attrState(type2, stream, state);
  }
  function attrContinuedState(type2, stream, state) {
    if (type2 == "string")
      return attrContinuedState;
    return attrState(type2, stream, state);
  }
  return {
    name: "xml",
    startState: function() {
      var state = {
        tokenize: inText,
        state: baseState,
        indented: 0,
        tagName: null,
        tagStart: null,
        context: null
      };
      return state;
    },
    token: function(stream, state) {
      if (!state.tagName && stream.sol())
        state.indented = stream.indentation();
      if (stream.eatSpace())
        return null;
      type = null;
      var style = state.tokenize(stream, state);
      if ((style || type) && style != "comment") {
        setStyle = null;
        state.state = state.state(type || style, stream, state);
        if (setStyle)
          style = setStyle;
      }
      return style;
    },
    indent: function(state, textAfter, cx) {
      var context = state.context;
      if (state.tokenize.isInAttribute) {
        if (state.tagStart == state.indented)
          return state.stringStartCol + 1;
        else
          return state.indented + cx.unit;
      }
      if (context && context.noIndent)
        return null;
      if (state.tokenize != inTag && state.tokenize != inText)
        return null;
      if (state.tagName) {
        if (config2.multilineTagIndentPastTag !== false)
          return state.tagStart + state.tagName.length + 2;
        else
          return state.tagStart + cx.unit * (config2.multilineTagIndentFactor || 1);
      }
      if (config2.alignCDATA && /<!\[CDATA\[/.test(textAfter))
        return 0;
      var tagAfter = textAfter && /^<(\/)?([\w_:\.-]*)/.exec(textAfter);
      if (tagAfter && tagAfter[1]) {
        while (context) {
          if (context.tagName == tagAfter[2]) {
            context = context.prev;
            break;
          } else if (config2.implicitlyClosed.hasOwnProperty(lower(context.tagName))) {
            context = context.prev;
          } else {
            break;
          }
        }
      } else if (tagAfter) {
        while (context) {
          var grabbers = config2.contextGrabbers[lower(context.tagName)];
          if (grabbers && grabbers.hasOwnProperty(lower(tagAfter[2])))
            context = context.prev;
          else
            break;
        }
      }
      while (context && context.prev && !context.startOfLine)
        context = context.prev;
      if (context)
        return context.indent + cx.unit;
      else
        return state.baseIndent || 0;
    },
    languageData: {
      indentOnInput: /<\/[\s\w:]+>$/,
      commentTokens: { block: { open: "<!--", close: "-->" } }
    },
    configuration: config2.htmlMode ? "html" : "xml",
    skipAttribute: function(state) {
      if (state.state == attrValueState)
        state.state = attrState;
    },
    xmlCurrentTag: function(state) {
      return state.tagName ? { name: state.tagName, close: state.type == "closeTag" } : null;
    },
    xmlCurrentContext: function(state) {
      var context = [];
      for (var cx = state.context; cx; cx = cx.prev)
        context.push(cx.tagName);
      return context.reverse();
    }
  };
}
mkXML({});
const html = mkXML({ htmlMode: true });
const htmlMode = html;
function last(array) {
  return array.at(-1);
}
function tokenUntil(stream, state, untilRegExp) {
  if (stream.sol()) {
    let indent = 0;
    for (; indent < state.indent; indent++) {
      if (!stream.eat(/\s/))
        break;
    }
    if (indent)
      return null;
  }
  let oldString = stream.string;
  let match = untilRegExp.exec(oldString.substr(stream.pos));
  if (match) {
    stream.string = oldString.substr(0, stream.pos + match.index);
  }
  function hideFirstChars(stream2, n, inner) {
    stream2.lineStart += n;
    try {
      return inner();
    } finally {
      stream2.lineStart -= n;
    }
  }
  let result = hideFirstChars(stream, state.indent, function() {
    return htmlMode.token(stream, state.htmlState);
  });
  stream.string = oldString;
  return result;
}
const config = {
  indentUnit: 2
};
const markdoc = {
  startState() {
    return {
      states: [],
      indent: 0,
      quoteKind: null,
      htmlState: htmlMode.startState(0)
    };
  },
  token(stream, state) {
    let match;
    switch (last(state.states)) {
      case void 0:
        if (stream.match(/^{%-?\s*comment\s*-?%}/)) {
          state.indent += config.indentUnit;
          state.states.push("comment");
          return "comment";
        } else if (stream.match(/^{%-?\s*raw\s*-?%}/)) {
          state.indent += config.indentUnit;
          state.states.push("raw");
          return "keyword";
        } else if (stream.match(/^{{/)) {
          state.states.push("object");
          return "keyword";
        } else if (match = stream.match(/^{%-?\s*(\w*)/)) {
          if (!match || typeof match == "boolean")
            return;
          if (match[1] == "endcase")
            state.indent -= 2 * config.indentUnit;
          else if (/^end/.test(match[1]))
            state.indent -= config.indentUnit;
          else if (match[1] == "case")
            state.indent += 2 * config.indentUnit;
          else if (/^(if|unless|for|tablerow)$/.test(match[1]))
            state.indent += config.indentUnit;
          state.states.push("tag");
          return "keyword";
        }
        return tokenUntil(stream, state, /{{|{%/);
      case "object":
      case "tag": {
        match = stream.match(/^['"]/);
        if (match && typeof match != "boolean") {
          state.states.push("string");
          state.quoteKind = match[0];
          return "string";
        } else if (stream.match(last(state.states) == "object" ? /^}}/ : /^%}/)) {
          state.states.pop();
        } else {
          stream.next();
        }
        return "keyword";
      }
      case "comment":
        if (stream.match(/^.*?{%-?\s*\/comment\s*-?%}/)) {
          state.indent -= config.indentUnit;
          state.states.pop();
        } else {
          stream.skipToEnd();
        }
        return "comment";
      case "raw":
        if (stream.match(/^{%-?\s*endraw\s*-?%}/)) {
          state.indent -= config.indentUnit;
          state.states.pop();
          return "keyword";
        }
        return tokenUntil(stream, state, /{%-?\s*endraw\s*-?%}/);
      case "string":
        match = stream.match(/^.*?(["']|\\[\s\S])/);
        if (!match) {
          stream.skipToEnd();
        } else if (typeof match == "boolean")
          ;
        else if (match[1] == state.quoteKind) {
          state.quoteKind = null;
          state.states.pop();
        }
        return "string";
    }
  },
  indent: function(state, textAfter, line) {
    let indent = state.indent, top = last(state.states);
    if (top == "comment" && /^{%-?\s*endcomment\s*-?%}/.test(textAfter))
      indent -= config.indentUnit;
    else if (top == "raw" && /^{%-?\s*endraw\s*-?%}/.test(textAfter))
      indent -= config.indentUnit;
    else if (top == void 0 && /^{%-?\s*endcase/.test(textAfter))
      indent -= 2 * config.indentUnit;
    else if (top == void 0 && /^{%-?\s*(when|end|els)/.test(textAfter))
      indent -= config.indentUnit;
    return indent + htmlMode.indent(state.htmlState, textAfter, line);
  }
};
export {
  markdoc
};
